@@ -102,17 +102,16 @@ local_folder_dispose (GObject *object)
 	folder = CAMEL_FOLDER (object);
 	local_folder = CAMEL_LOCAL_FOLDER (object);
 
-	if (folder->summary) {
+	if (camel_folder_get_folder_summary (folder)) {
 		/* Something can hold the reference to the folder longer than
 		   the parent store is alive, thus count with it. */
 		if (camel_folder_get_parent_store (folder)) {
 			camel_local_summary_sync (
-				CAMEL_LOCAL_SUMMARY (folder->summary),
+				CAMEL_LOCAL_SUMMARY (camel_folder_get_folder_summary (folder)),
 				FALSE, local_folder->changes, NULL, NULL);
 		}
 	}
 
-	g_clear_object (&folder->summary);
 	g_clear_object (&local_folder->search);
 	g_clear_object (&local_folder->index);
 
@@ -180,39 +179,39 @@ local_folder_constructed (GObject *object)
 	path = g_strdup_printf ("%s/%s", root_path, full_name);
 
 	if ((tmp = getenv ("HOME")) && strncmp (tmp, path, strlen (tmp)) == 0)
-		/* Translators: This is used for a folder description,
-		 * for folders being under $HOME.  The first %s is replaced
-		 * with a relative path under $HOME, the second %s is
-		 * replaced with a protocol name, like mbox/maldir/... */
 		description = g_strdup_printf (
+			/* Translators: This is used for a folder description,
+			 * for folders being under $HOME.  The first %s is replaced
+			 * with a relative path under $HOME, the second %s is
+			 * replaced with a protocol name, like mbox/maldir/... */
 			_("~%s (%s)"),
 			path + strlen (tmp),
 			provider->protocol);
 	else if ((tmp = "/var/spool/mail") && strncmp (tmp, path, strlen (tmp)) == 0)
-		/* Translators: This is used for a folder description, for
-		 * folders being under /var/spool/mail.  The first %s is
-		 * replaced with a relative path under /var/spool/mail,
-		 * the second %s is replaced with a protocol name, like
-		 * mbox/maldir/... */
 		description = g_strdup_printf (
+			/* Translators: This is used for a folder description, for
+			 * folders being under /var/spool/mail.  The first %s is
+			 * replaced with a relative path under /var/spool/mail,
+			 * the second %s is replaced with a protocol name, like
+			 * mbox/maldir/... */
 			_("mailbox: %s (%s)"),
 			path + strlen (tmp),
 			provider->protocol);
 	else if ((tmp = "/var/mail") && strncmp (tmp, path, strlen (tmp)) == 0)
-		/* Translators: This is used for a folder description, for
-		 * folders being under /var/mail.  The first %s is replaced
-		 * with a relative path under /var/mail, the second %s is
-		 * replaced with a protocol name, like mbox/maldir/... */
 		description = g_strdup_printf (
+			/* Translators: This is used for a folder description, for
+			 * folders being under /var/mail.  The first %s is replaced
+			 * with a relative path under /var/mail, the second %s is
+			 * replaced with a protocol name, like mbox/maldir/... */
 			_("mailbox: %s (%s)"),
 			path + strlen (tmp),
 			provider->protocol);
 	else
-		/* Translators: This is used for a folder description.
-		 * The first %s is replaced with a folder's full path,
-		 * the second %s is replaced with a protocol name, like
-		 * mbox/maldir/... */
 		description = g_strdup_printf (
+			/* Translators: This is used for a folder description.
+			 * The first %s is replaced with a folder's full path,
+			 * the second %s is replaced with a protocol name, like
+			 * mbox/maldir/... */
 			_("%s (%s)"), path,
 			provider->protocol);
 
@@ -223,6 +222,18 @@ local_folder_constructed (GObject *object)
 	g_free (path);
 }
 
+static guint32
+local_folder_get_permanent_flags (CamelFolder *folder)
+{
+	return CAMEL_MESSAGE_ANSWERED |
+		CAMEL_MESSAGE_DELETED |
+		CAMEL_MESSAGE_DRAFT |
+		CAMEL_MESSAGE_FLAGGED |
+		CAMEL_MESSAGE_SEEN |
+		CAMEL_MESSAGE_ANSWERED_ALL |
+		CAMEL_MESSAGE_USER;
+}
+
 static GPtrArray *
 local_folder_search_by_expression (CamelFolder *folder,
                                    const gchar *expression,
@@ -331,8 +342,8 @@ local_folder_rename (CamelFolder *folder,
 	g_free (statepath);
 
 	/* FIXME: Poke some internals, sigh */
-	g_free (((CamelLocalSummary *) folder->summary)->folder_path);
-	((CamelLocalSummary *) folder->summary)->folder_path = g_strdup (lf->folder_path);
+	g_free (((CamelLocalSummary *) camel_folder_get_folder_summary (folder))->folder_path);
+	((CamelLocalSummary *) camel_folder_get_folder_summary (folder))->folder_path = g_strdup (lf->folder_path);
 
 	CAMEL_FOLDER_CLASS (camel_local_folder_parent_class)->rename (folder, newname);
 }
@@ -399,7 +410,7 @@ local_folder_refresh_info_sync (CamelFolder *folder,
 		CAMEL_LOCAL_STORE (parent_store));
 
 	if (need_summary_check &&
-	    camel_local_summary_check ((CamelLocalSummary *) folder->summary, lf->changes, cancellable, error) == -1)
+	    camel_local_summary_check ((CamelLocalSummary *) camel_folder_get_folder_summary (folder), lf->changes, cancellable, error) == -1)
 		return FALSE;
 
 	if (camel_folder_change_info_changed (lf->changes)) {
@@ -428,7 +439,7 @@ local_folder_synchronize_sync (CamelFolder *folder,
 
 	/* if sync fails, we'll pass it up on exit through ex */
 	success = (camel_local_summary_sync (
-		(CamelLocalSummary *) folder->summary,
+		(CamelLocalSummary *) camel_folder_get_folder_summary (folder),
 		expunge, lf->changes, cancellable, error) == 0);
 	camel_local_folder_unlock (lf);
 
@@ -470,6 +481,7 @@ camel_local_folder_class_init (CamelLocalFolderClass *class)
 	object_class->constructed = local_folder_constructed;
 
 	folder_class = CAMEL_FOLDER_CLASS (class);
+	folder_class->get_permanent_flags = local_folder_get_permanent_flags;
 	folder_class->search_by_expression = local_folder_search_by_expression;
 	folder_class->search_by_uids = local_folder_search_by_uids;
 	folder_class->search_free = local_folder_search_free;
@@ -504,14 +516,8 @@ camel_local_folder_init (CamelLocalFolder *local_folder)
 	local_folder->priv = CAMEL_LOCAL_FOLDER_GET_PRIVATE (local_folder);
 	g_mutex_init (&local_folder->priv->search_lock);
 
-	folder->folder_flags |= CAMEL_FOLDER_HAS_SUMMARY_CAPABILITY;
+	camel_folder_set_flags (folder, camel_folder_get_flags (folder) | CAMEL_FOLDER_HAS_SUMMARY_CAPABILITY);
 
-	folder->permanent_flags = CAMEL_MESSAGE_ANSWERED |
-	    CAMEL_MESSAGE_DELETED | CAMEL_MESSAGE_DRAFT |
-	    CAMEL_MESSAGE_FLAGGED | CAMEL_MESSAGE_SEEN |
-	    CAMEL_MESSAGE_ANSWERED_ALL | CAMEL_MESSAGE_USER;
-
-	folder->summary = NULL;
 	local_folder->search = NULL;
 }
 
@@ -538,6 +544,7 @@ camel_local_folder_construct (CamelLocalFolder *lf,
 	CamelStore *parent_store;
 	const gchar *full_name;
 	gboolean need_summary_check;
+	gboolean filter_all = FALSE, filter_junk = TRUE;
 
 	folder = CAMEL_FOLDER (lf);
 	full_name = camel_folder_get_full_name (folder);
@@ -549,12 +556,21 @@ camel_local_folder_construct (CamelLocalFolder *lf,
 
 	local_settings = CAMEL_LOCAL_SETTINGS (settings);
 	lf->base_path = camel_local_settings_dup_path (local_settings);
+	filter_all = camel_local_settings_get_filter_all (local_settings);
+	filter_junk = camel_local_settings_get_filter_junk (local_settings);
 
 	g_object_unref (settings);
 
 	ls = CAMEL_LOCAL_STORE (parent_store);
 	need_summary_check = camel_local_store_get_need_summary_check (ls);
 
+	filter_junk = filter_junk || camel_local_store_is_main_store (CAMEL_LOCAL_STORE (parent_store));
+	if (filter_all || filter_junk) {
+		camel_folder_set_flags (folder, camel_folder_get_flags (folder) |
+			(filter_all ? CAMEL_FOLDER_FILTER_RECENT : 0) |
+			(filter_junk ? CAMEL_FOLDER_FILTER_JUNK : 0));
+	}
+
 	lf->folder_path = camel_local_store_get_full_path (ls, full_name);
 	lf->index_path = camel_local_store_get_meta_path (ls, full_name, ".ibex");
 	statepath = camel_local_store_get_meta_path (ls, full_name, ".cmeta");
@@ -622,13 +638,13 @@ camel_local_folder_construct (CamelLocalFolder *lf,
 		forceindex = FALSE;
 	}
 
-	folder->summary = (CamelFolderSummary *) CAMEL_LOCAL_FOLDER_GET_CLASS (lf)->create_summary (lf, lf->folder_path, lf->index);
-	if (!(flags & CAMEL_STORE_IS_MIGRATING) && !camel_local_summary_load ((CamelLocalSummary *) folder->summary, forceindex, NULL)) {
+	camel_folder_take_folder_summary (folder, CAMEL_FOLDER_SUMMARY (CAMEL_LOCAL_FOLDER_GET_CLASS (lf)->create_summary (lf, lf->folder_path, lf->index)));
+	if (!(flags & CAMEL_STORE_IS_MIGRATING) && !camel_local_summary_load ((CamelLocalSummary *) camel_folder_get_folder_summary (folder), forceindex, NULL)) {
 		/* ? */
 		if (need_summary_check &&
-		    camel_local_summary_check ((CamelLocalSummary *) folder->summary, lf->changes, cancellable, error) == 0) {
+		    camel_local_summary_check ((CamelLocalSummary *) camel_folder_get_folder_summary (folder), lf->changes, cancellable, error) == 0) {
 			/* we sync here so that any hard work setting up the folder isn't lost */
-			if (camel_local_summary_sync ((CamelLocalSummary *) folder->summary, FALSE, lf->changes, cancellable, error) == -1) {
+			if (camel_local_summary_sync ((CamelLocalSummary *) camel_folder_get_folder_summary (folder), FALSE, lf->changes, cancellable, error) == -1) {
 				g_object_unref (folder);
 				return NULL;
 			}
@@ -713,11 +729,11 @@ set_cannot_get_message_ex (GError **error,
                            const gchar *folder_path,
                            const gchar *detailErr)
 {
-	/* Translators: The first %s is replaced with a message ID,
-	 * the second %s is replaced with the folder path,
-	 * the third %s is replaced with a detailed error string */
 	g_set_error (
 		error, CAMEL_ERROR, err_code,
+		/* Translators: The first %s is replaced with a message ID,
+		 * the second %s is replaced with the folder path,
+		 * the third %s is replaced with a detailed error string */
 		_("Cannot get message %s from folder %s\n%s"),
 		msgID, folder_path, detailErr);
 }
